/*
 die drei klassen:
   Punch::Card
   Punch::CardColumn
   Punch::Bit
*/

#include <iostream>
#include <list>

typedef unsigned char byte_t;

namespace Punch {

class CardBit {
public:
	// the common meaing of these bits:
	static const bool hole = true;
	static const bool no_hole = false;
	static const bool default_value = false;

	// well, this is the real content: :-)
	bool value;

	CardBit() : value(default_value) {}
	CardBit(bool value) : value(value) {}
	CardBit(int value) { this->value = value?true:false; }

	void set(bool value) { this->value = value; }
	bool get() { return value; }

	void invert() { value = !value; }

	// wird von bool casting uebernommen
	//CardBit operator!() { return CardBit(!value); }
	CardBit& operator=(const CardBit& a) {
		value=a.value; return *this; }
	operator bool() const { return value; }
};

// to make sure:
static const int  hole_1 =  1;
static const int  hole_2 =  2;
static const int  hole_3 =  3;
static const int  hole_4 =  4;
static const int  hole_5 =  5;
static const int  hole_6 =  6;
static const int  hole_7 =  7;
static const int  hole_8 =  8;
static const int  hole_9 =  9;
static const int  hole_0 =  0;
static const int hole_11 = 10;
static const int hole_12 = 11;

class CardWord {
public:
	static const int word_length = 12;
	Punch::CardBit bits[ word_length ];

	// getters and setters
	void set(int pos, const CardBit& value) { bits[pos]=value; }
	void set(const CardBit&  bit0,
		 const CardBit&  bit1,
		 const CardBit&  bit2,
		 const CardBit&  bit3,
		 const CardBit&  bit4,
		 const CardBit&  bit5,
		 const CardBit&  bit6,
		 const CardBit&  bit7,
		 const CardBit&  bit8,
		 const CardBit&  bit9,
		 const CardBit& bit10,
		 const CardBit& bit11) {
		bits[0] = bit0; bits[1] = bit1; bits[2] = bit2;
		bits[3] = bit3; bits[4] = bit4; bits[5] = bit5;
		bits[6] = bit6; bits[7] = bit7; bits[8] = bit8;
		bits[9] = bit9; bits[10] = bit10; bits[11] = bit11;
	}
	CardBit& get(int pos) {
		if(pos<0||pos>word_length) throw "Out of range";
		return bits[pos]; }

	void print(std::ostream& os) const;

	CardBit& operator[](int x) { return get(x); }

	const CardWord operator&(const CardWord& right) const {
		CardWord result = *this;
		for(int x=0; x<word_length; x++)
			result[x].value &= right.bits[x].value;
		return result;
	}

	const CardWord operator|(const CardWord& right) const {
		CardWord result = *this;
		for(int x=0; x<word_length; x++)
			result[x].value |= right.bits[x].value;
		return result;
	}

	const CardWord operator^(const CardWord& right) const {
		CardWord result = *this;
		for(int x=0; x<word_length; x++)
			result[x].value ^= right.bits[x].value;
		return result;
	}

	const CardWord operator<<(const int& right) const {
		CardWord result;
		// source, target fields
		for(int s=0,t=-right; s<word_length; s++,t++) {
			// make it cyclic
			if(t < 0) t += word_length;
			if(t >= word_length) t -= word_length;
			result[t] = bits[s];
		}
		return result;
	}

	const CardWord operator>>(const int& right) const {
		return operator<<(-right);
	}

}; // class CardWord
/*
class CardWordIterator {
	int current_index;
	CardWord* card;

	CardWordIterator next();
	CardWordIterator previous();
}
*/

class Card {
public:
	static const int col_length = 80;
	Punch::CardWord word[ col_length ];

	// initialize blanco card
	Card() {}
	// read in card from some stream
	Card(CardFormat format, std::istream& is) {
		this->read(format, is); }

	// this will copy your word
	void set(int col, const CardWord& word) {
		this->word[col] = word; }

	// iteratoren
	//CardWordIterator start();
	//CardWordIterator end();

	void print(std::ostream& os) const;
};

typedef enum {
	JONES_FORMAT,
	ASCII_FORMAT
} CardFormat;

class CardFile {
public:
	public list<Punch::Card> cards;
	void write(CardFormat format, std::ostream& os);
	bool read(CardFormat format, std::istream& is);
}


} // namespace
