#include <cstdio>
#include <sstream>

#include "card.h"


PunchCard::Column::Column() {
	// yeah, empty:
	this->lower = 0;
	this->upper = 0;
}

PunchCard::Card::Card() {
	// nothing to say here.
}

bool PunchCard::Column::get_value_at(int pos) {
	switch(pos) {
		case 0: // lower bit 0
		case 1: // lower bit 1
		case 2: // lower bit 2
		case 3: // lower bit 3
		case 4: // lower bit 4
		case 5: // lower bit 5
		case 6: // lower bit 6
		case 7: // lower bit 7
			return (this->lower >> pos) & 0x01 != 0;
		case 8:  // upper bit 0
		case 9:  // upper bit 1
			return (this->upper >> (pos-8)) & 0x01 != 0;
		case 11: // upper bit 2
		case 12: // upper bit 3
			return (this->upper >> (pos-9)) & 0x01 != 0;
		default:
			printf("Column get: Index %d out of range!\n", pos);
			return true; // bla, eigntl. exception
	}
}

void PunchCard::Column::set_value_at(int pos, bool value) {
	switch(pos) {
		case 0: // lower bit 0
		case 1: // lower bit 1
		case 2: // lower bit 2
		case 3: // lower bit 3
		case 4: // lower bit 4
		case 5: // lower bit 5
		case 6: // lower bit 6
		case 7: // lower bit 7
			this->lower |= 0x01 << pos;
			return;
		case 8:  // upper bit 0
		case 9:  // upper bit 1
			this->upper |= 0x01 << (pos-8);
			return;
		case 11: // upper bit 2
		case 12: // upper bit 3
			this->upper |= 0x01 << (pos-9);
			return;
		default:
			printf("Column set: Index %d out of range!\n", pos);
			return;
	}
}

void PunchCard::Card::row_to_string(std::string& r, int row) {
	// helper function
	for(int x=0; x < this->col_length; x++) {
		r += this->columns[x].get_value_at(row) ? "0" : " ";
	}
}

std::string* PunchCard::Card::to_string() {
/*
typical output like:
     ________________________________________________________________
    / [ row for labeling ]
12 / O           OOOOOOOOO                        OOOOOO
11|   O                   OOOOOOOOO                     OOOOOO
 0|    O                          OOOOOOOO      ?     ?     OOOOOO
 1|     O        O        O        O
 2|      O        O        O        O       O     ?     ?     O
 3|       O        O        O        O       O     O     O     O
 4|        O        O        O        O       O     O     O     O
 5|         O        O        O        O       O     O     O     O
 6|          O        O        O        O       O     O     O     O
 7|           O        O        O        O       O     O     O     O
 8|            O        O        O        O O OOOOOOOOOOOOOOOOOOOOOOOO
 9|             O        O        O        O
  |__|________|________|________|____________________________________
     1        10       20       30       ...
*/
	std::string* r = new std::string();
	*r += "     ________________________________________________________________\n";
	*r += "    / [ row for labeling ]\n";
	*r += "12 / ";
	this->row_to_string(*r, 12);
	*r += "\n";
	*r += "11|  ";
	this->row_to_string(*r, 11);
	*r += "\n";
	for(int x=0; x < 10; x++) {
		// haessliche Loesung, x an string zu haengen
		std::ostringstream temp;
		temp << x;
		*r += " ";
		*r += temp.str();
		*r += "|  ";
		this->row_to_string(*r, x);
		*r += "\n";
	}
	*r += "  |__|________|________|________|____________________________________\n";
	*r += "     1        10       20       30       ...\n";
	
	return r;
}
